"use strict";
var zlib_1 = require("zlib");
var querystring_1 = require("querystring");
var getRawBody = require("raw-body");
var createError = require("http-errors");
var DEFAULT_LIMIT = 100 * 1024; // 100kb.
var DEFAULT_ENCODING = 'utf8';
// Allowed whitespace is defined in RFC 7159.
// http://www.rfc-editor.org/rfc/rfc7159.txt
var STRICT_JSON_REGEXP = /^[\x20\x09\x0a\x0d]*[\[\{]/;
/**
 * Supported encodings.
 */
exports.decoders = {
    deflate: function (stream) { return stream.pipe(zlib_1.createInflate()); },
    gzip: function (stream) { return stream.pipe(zlib_1.createGunzip()); }
};
/**
 * Default JSON parser.
 */
function jsonParse(str) {
    if (str.length === 0) {
        return {};
    }
    if (!STRICT_JSON_REGEXP.test(str)) {
        throw createError(400, 'Invalid JSON, only supports object and array');
    }
    return JSON.parse(str);
}
/**
 * Default support content types.
 */
exports.jsonTypes = ['application/json'];
exports.formTypes = ['application/x-www-form-urlencoded'];
exports.textTypes = ['text/plain', 'text/html'];
/**
 * Parse any incoming stream.
 */
function parse(stream, headers, options) {
    if (options === void 0) { options = {}; }
    var type = getType(headers);
    if (!type) {
        return Promise.reject(createError(415, 'Missing content-type'));
    }
    var jsonType = options.jsonTypes || exports.jsonTypes;
    if (jsonType.indexOf(type) > -1) {
        return json(stream, headers, options);
    }
    var formType = options.formTypes || exports.formTypes;
    if (formType.indexOf(type) > -1) {
        return form(stream, headers, options);
    }
    var textType = options.textTypes || exports.textTypes;
    if (textType.indexOf(type) > -1) {
        return text(stream, headers, options);
    }
    return Promise.reject(createError(415, "Unsupported content-type: " + type));
}
exports.parse = parse;
/**
 * Parse the stream as JSON.
 */
function json(stream, headers, options) {
    if (options === void 0) { options = {}; }
    return text(stream, headers, options).then(options.jsonParse || jsonParse);
}
exports.json = json;
/**
 * Parse the stream as a form.
 */
function form(stream, headers, options) {
    if (options === void 0) { options = {}; }
    return text(stream, headers, options).then(options.formParse || querystring_1.parse);
}
exports.form = form;
/**
 * Parse the stream as text.
 */
function text(stream, headers, options) {
    if (options === void 0) { options = {}; }
    var _a = inflate(stream, headers, options.decoders), err = _a[0], req = _a[1], length = _a[2];
    if (err) {
        return Promise.reject(err);
    }
    var limit = options.limit || DEFAULT_LIMIT;
    var encoding = options.encoding || DEFAULT_ENCODING;
    return getRawBody(req, { length: length, limit: limit, encoding: encoding });
}
exports.text = text;
/**
 * Inflate the readable stream.
 */
function inflate(stream, headers, encoding) {
    if (encoding === void 0) { encoding = exports.decoders; }
    var enc = String(headers['content-encoding'] || 'identity').toLowerCase();
    if (enc === 'identity') {
        return [undefined, stream, ~~headers['content-length']];
    }
    if (encoding[enc]) {
        return [undefined, encoding[enc](stream), undefined];
    }
    return [createError("Unsupport content-encoding: " + enc), stream, undefined];
}
exports.inflate = inflate;
/**
 * Return the `content-type` string.
 */
function getType(headers) {
    var contentType = headers['content-type'];
    if (!contentType) {
        return;
    }
    if (Array.isArray(contentType)) {
        contentType = contentType[0];
    }
    var index = contentType.indexOf(';');
    return index > -1 ? contentType.substr(0, index) : contentType;
}
exports.getType = getType;
//# sourceMappingURL=index.js.map